@@ -1,6 +1,14 @@
 import sys
 import hashlib
-
+try:
+    # this will work with micropython and python < 3.10
+    # but will raise and exception if ripemd is not supported (python3.10, openssl 3)
+    hashlib.new('ripemd160')
+    def ripemd160(msg):
+        return hashlib.new('ripemd160', msg).digest()
+except:
+    # otherwise use pure python implementation
+    from .util.py_ripemd160 import ripemd160
 
 def double_sha256(msg):
     """sha256(sha256(msg)) -> bytes"""
@@ -9,7 +17,7 @@ def double_sha256(msg):
 
 def hash160(msg):
     """ripemd160(sha256(msg)) -> bytes"""
-    return hashlib.new('ripemd160', hashlib.sha256(msg).digest()).digest()
+    return ripemd160(hashlib.sha256(msg).digest())
 
 
 def sha256(msg):
@@ -17,11 +25,6 @@ def sha256(msg):
     return hashlib.sha256(msg).digest()
 
 
-def ripemd160(msg):
-    """one-line rmd160(msg) -> bytes"""
-    return hashlib.new('ripemd160', msg).digest()
-
-
 def tagged_hash(tag: str, data: bytes) -> bytes:
     """BIP-Schnorr tag-specific key derivation"""
     hashtag = hashlib.sha256(tag.encode()).digest()
